use Test::Pod::Coverage tests => 4;

pod_coverage_ok( 'Rsync::Config::Exceptions', 'Rsync::Config::Exceptions is covered');
pod_coverage_ok( 'Rsync::Config::Atom'      , 'Rsync::Config::Atom is covered');
pod_coverage_ok( 'Rsync::Config::Module'    , 'Rsync::Config::Module is covered');
pod_coverage_ok( 'Rsync::Config'            , 'Rsync::Config is covered');
